/*
 * steer.h - header for defining a new steering module
 *
 * Copyright (C) 2022 IOPSYS Software Solutions AB. All rights reserved.
 *
 * Author: anjan.chanda@iopsys.eu
 *
 * See LICENSE file for license related information.
 *
 */

#ifndef STEER_H
#define STEER_H

#include <stdint.h>
#include <libubox/list.h>

#include "cntlr.h"
#ifdef __cplusplus
extern "C" {
#endif

#define DEFAULT_RCPI_THRESHOLD 86

enum steer_verdict {
	STEER_VERDICT_UNDECIDED,
	STEER_VERDICT_OK,
	STEER_VERDICT_MAYBE,
	STEER_VERDICT_NOK,
	STEER_VERDICT_EXCLUDE,
};

enum steer_reason {
	STEER_REASON_UNDEFINED,
	STEER_REASON_LOW_RCPI,
	STEER_REASON_LOW_THPUT,
	STEER_REASON_HIGH_PER,
	STEER_REASON_OTHER,
};

typedef enum steer_verdict steer_verdict_t;
typedef enum steer_reason steer_reason_t;

struct steer_config {
	uint8_t rcpi_threshold;
	uint8_t rcpi_hysteresis;
	uint8_t rcpi_diffsnr;
	uint8_t ch_utilization;
};

struct steer_target {
	uint8_t opclass;
	uint8_t channel;
	uint8_t rcpi;
	uint8_t rsni;
	uint8_t bssid[6];
	bool requested; /* beacon metrics query sent by cntlr */
	bool stale;	/* outdated report got before most recent request sent */
	struct list_head list;
};

struct steer_sta {
	struct sta *s;
	struct list_head *nbrlist;
	struct list_head *bcnlist;
	steer_verdict_t verdict;
	struct steer_target *best;
	steer_reason_t reason;
	uint32_t mode;
};

struct steer_control {
	char name[64];
	//uint8_t rcpi_threshold;
	//uint8_t rcpi_hysteresis;
	uint8_t cbinterval;
	void *priv;
	int (*init)(void **);
	int (*exit)(void *);
	int (*steer)(void *, struct steer_sta *candidate);
	int (*config)(void *, struct steer_config *);
	void *handle;
	struct list_head list;
};


#ifdef __cplusplus
}
#endif

#endif /* STEER_H */
